local L = AceLibrary("AceLocale-2.2"):new("VersionChecker")
VersionChecker = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "FuBarPlugin-2.0")
VersionChecker.revision = tonumber(string.sub("$Revision: 31954 $", 12, -3))
VersionChecker.version = "r" .. VersionChecker.revision
VersionChecker.date = string.sub("$Date: 2007-04-06 00:30:07 -0400 (Fri, 06 Apr 2007) $", 8, 17)
VersionChecker.hasIcon = "Interface\\Icons\\Spell_Shadow_Charm"

local AceComm = AceLibrary("AceComm-2.0")
local Tablet = AceLibrary("Tablet-2.0")
local RollCall = AceLibrary("RollCall-1.0")

function VersionChecker:OnInitialize()
	self:RegisterDB("VersionCheckerDB")
	self:RegisterDefaults("profile", {
	})
end

function VersionChecker:OnEnable(first)
	if first then
		AceComm:RegisterAddonVersionReceptor(self, "OnAddonVersion")
	end
end

local currentAddon
local distribution
local currentPerson = nil
local data = {}

function VersionChecker:OnAddonVersion(player, addon, version)
	if not self:IsActive() or addon ~= currentAddon then
		return
	end

	data[player] = version

	self:UpdateTooltip()
end

local function mysort(a, b)
	local alpha = data[a]
	local bravo = data[b]
	if bravo == UNKNOWN then
		if alpha == UNKNOWN then
			return a < b
		end
		return true
	elseif alpha == UNKNOWN then
		return false
	elseif not bravo then
		if not alpha then
			return a < b
		end
		return true
	elseif not alpha then
		return false
	elseif bravo == true then
		if alpha == true then
			return a < b
		end
		return true
	elseif alpha == true then
		return false
	end
	if type(alpha) ~= "number" or type(bravo) ~= "number" then
		alpha = tostring(alpha):lower():gsub("%s", "")
		bravo = tostring(bravo):lower():gsub("%s", "")
		if alpha:len() ~= bravo:len() then
			return alpha:len() > bravo:len()
		end
	end	
	if alpha == bravo then
		return a < b
	end
	return alpha > bravo
end
local tmp = {}
function VersionChecker:OnTooltipUpdate()
	Tablet:AddCategory('columns', 2):AddLine('text', currentAddon or NONE, 'text2', distribution == PLAYER and currentPerson or distribution)
	local cat = Tablet:AddCategory(
		'columns', 2,
		'text', "Player",
		'text2', "Version"
	)
	local latest
	for k,v in pairs(data) do
		if v ~= UNKNOWN and (type(v) == "string" or type(v) == "number") and (not latest or latest < v) then
			latest = v
		end
		tmp[#tmp+1] = k
	end
	table.sort(tmp, mysort)
	for i,k in ipairs(tmp) do
		tmp[i] = nil
		local v = data[k]
		local r, g, b = 1, 1, 0
		if not v then
			g = 0
		elseif v == latest then
			r = 0
		elseif v == UNKNOWN then
			r, g, b = 0.8, 0.8, 0.8
		end
		if v == true then
			v = YES
		elseif not v then
			v = NONE
		end
		cat:AddLine(
			'text', k,
			'textR', k == UnitName('player') and 0 or 1,
			'textG', 1,
			'textB', 0,
			'text2', tostring(v),
			'text2R', r,
			'text2G', g,
			'text2B', b
		)
	end
end

VersionChecker.OnMenuRequest = {
	type = 'group',
	args = {
		guild = {
			type = 'text',
			name = GUILD,
			desc = GUILD,
			get = false,
			set = function(text)
				currentAddon = text
				distribution = GUILD
				for k in pairs(data) do
					data[k] = nil
				end
				for name in RollCall:GetIterator() do
					data[name] = UNKNOWN
				end
				AceComm:QueryAddonVersion(text, "GUILD")
				AceComm:QueryAddonVersion(text, "WHISPER", UnitName('player'))
			end,
			disabled = function()
				return not IsInGuild()
			end,
			usage = '<Addon/library name>'
		},
		group = {
			type = 'text',
			name = GROUP,
			desc = GROUP,
			get = false,
			set = function(text)
				currentAddon = text
				distribution = GROUP
				for k in pairs(data) do
					data[k] = nil
				end
				if GetNumRaidMembers() > 0 then
					for i = 1, GetNumRaidMembers() do
						data[UnitName("raid" .. i)] = UNKNOWN
					end
				else
					for i = 1, GetNumPartyMembers() do
						data[UnitName("party" .. i)] = UNKNOWN
					end
				end
				AceComm:QueryAddonVersion(text, "GROUP")
				AceComm:QueryAddonVersion(text, "WHISPER", UnitName('player'))
			end,
			disabled = function()
				return GetNumPartyMembers() == 0
			end,
			usage = '<Addon/library name>'
		},
		person = {
			type = 'group',
			name = PLAYER,
			desc = PLAYER,
			args = {
				addon = {
					type = 'text',
					name = "Addon",
					desc = "Addon",
					get = false,
					set = function(text)
						currentAddon = text
						distribution = PLAYER
						for k in pairs(data) do
							data[k] = nil
						end
						data[text] = UNKNOWN
						data[UnitName('player')] = UNKNOWN
						AceComm:QueryAddonVersion(text, "WHISPER", currentPerson)
						AceComm:QueryAddonVersion(text, "WHISPER", UnitName('player'))
					end,
					validate = function(text)
						return currentPerson
					end,
					usage = '<Addon/library name>'
				},
				name = {
					type = 'text',
					name = PLAYER,
					desc = PLAYER,
					get = function()
						return currentPerson
					end,
					set = function(text)
						currentPerson = text
					end,
					usage = '<Player name>'
				}
			}
		}
	}
}